#ifndef NODE_H
#define NODE_H

#include <cstdio>
#include <string>
#include <map>

class Node
{
public:
	Node(void) : m_left(nullptr), m_right(nullptr)
	{
	}

	Node(Node *left, Node *right) : m_left(left), m_right(right)
	{
	}

	static bool compare(const Node *lhs, const Node *rhs)
	{
		return lhs->getValue() > rhs->getValue();
	}

	virtual unsigned getValue(void) const
	{
		unsigned value = 0;

		if (m_left)
			value += m_left->getValue();
		if (m_right)
			value += m_right->getValue();

		return value;
	}

	const Node *getLeftChild(void) const
	{
		return m_left;
	}

	const Node *getRightChild(void) const
	{
		return m_right;
	}

	virtual void print(unsigned level) const
	{
		for (unsigned i = 0; i < level; i++)
			printf("    ");
		printf("[v = %i]\n", getValue());
	}

	virtual void addCode(std::map<char, std::string> &dict, std::string code) const
	{
		if (m_left)
			m_left->addCode(dict, code + '0');
		if (m_right)
			m_right->addCode(dict, code + '1');
	}

private:
	Node *m_left;
	Node *m_right;
};

#endif
